@@ -8,7 +8,7 @@ Created: 2022-11-09 [2.0rc2] RecentProjects
 Created: 2024-06-16 [2.5rc1] RecentPaths
 
 This file is a part of novelWriter
-Copyright 2018–2024, Veronica Berglyd Olsen
+Copyright (C) 2018 Veronica Berglyd Olsen and novelWriter contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@ import sys
 from datetime import datetime
 from pathlib import Path
 from time import time
+from typing import TYPE_CHECKING
 
 from PyQt5.QtCore import (
     PYQT_VERSION, PYQT_VERSION_STR, QT_VERSION, QT_VERSION_STR, QLibraryInfo,
@@ -40,10 +41,16 @@ from PyQt5.QtCore import (
 from PyQt5.QtGui import QFont, QFontDatabase
 from PyQt5.QtWidgets import QApplication
 
-from novelwriter.common import NWConfigParser, checkInt, checkPath, describeFont, formatTimeStamp
+from novelwriter.common import (
+    NWConfigParser, checkInt, checkPath, describeFont, fontMatcher,
+    formatTimeStamp
+)
 from novelwriter.constants import nwFiles, nwUnicode
 from novelwriter.error import formatException, logException
 
+if TYPE_CHECKING:  # pragma: no cover
+    from novelwriter.core.projectdata import NWProjectData
+
 logger = logging.getLogger(__name__)
 
 
@@ -98,8 +105,11 @@ class Config:
         self._qtTrans = {}
 
         # PDF Manual
-        pdfDocs = self._appPath / "assets" / "manual.pdf"
-        self.pdfDocs = pdfDocs if pdfDocs.is_file() else None
+        self._manuals: dict[str, Path] = {}
+        if (assets := self._appPath / "assets").is_dir():
+            for item in assets.iterdir():
+                if item.is_file() and item.stem.startswith("manual") and item.suffix == ".pdf":
+                    self._manuals[item.stem] = item
 
         # User Settings
         # =============
@@ -109,7 +119,7 @@ class Config:
 
         # General GUI Settings
         self.guiLocale    = self._qLocale.name()
-        self.guiTheme     = "default"        # GUI theme
+        self.guiTheme     = "default_light"  # GUI theme
         self.guiSyntax    = "default_light"  # Syntax theme
         self.guiFont      = QFont()          # Main GUI font
         self.guiScale     = 1.0              # Set automatically by Theme class
@@ -132,6 +142,7 @@ class Config:
         self.emphLabels      = True   # Add emphasis to H1 and H2 item labels
         self.backupOnClose   = False  # Flag for running automatic backups
         self.askBeforeBackup = True   # Flag for asking before running automatic backup
+        self.askBeforeExit   = True   # Flag for asking before exiting the app
 
         # Text Editor Settings
         self.textFont        = QFont()  # Editor font
@@ -161,8 +172,9 @@ class Config:
 
         self.dialogStyle     = 2        # Quote type to use for dialogue
         self.allowOpenDial   = True     # Allow open-ended dialogue quotes
-        self.narratorBreak   = ""       # Symbol to use for narrator break
         self.dialogLine      = ""       # Symbol to use for dialogue line
+        self.narratorBreak   = ""       # Symbol to use for narrator break
+        self.narratorDialog  = ""       # Symbol for alternating between dialogue and narrator
         self.altDialogOpen   = ""       # Alternative dialog symbol, open
         self.altDialogClose  = ""       # Alternative dialog symbol, close
         self.highlightEmph   = True     # Add colour to text emphasis
@@ -190,6 +202,7 @@ class Config:
         # State
         self.showViewerPanel = True   # The panel for the viewer is visible
         self.showEditToolBar = False  # The document editor toolbar visibility
+        self.showSessionTime = True   # Show the session time in the status bar
         self.viewComments    = True   # Comments are shown in the viewer
         self.viewSynopsis    = True   # Synopsis is shown in the viewer
 
@@ -252,6 +265,14 @@ class Config:
     def hasError(self) -> bool:
         return self._hasError
 
+    @property
+    def pdfDocs(self) -> Path | None:
+        return self._manuals.get(f"manual_{self.locale.name()}", self._manuals.get("manual"))
+
+    @property
+    def locale(self) -> QLocale:
+        return self._dLocale
+
     @property
     def recentProjects(self) -> RecentProjects:
         return self._recentProjects
@@ -364,10 +385,11 @@ class Config:
     def setGuiFont(self, value: QFont | str | None) -> None:
         """Update the GUI's font style from settings."""
         if isinstance(value, QFont):
-            self.guiFont = value
+            self.guiFont = fontMatcher(value)
         elif value and isinstance(value, str):
-            self.guiFont = QFont()
-            self.guiFont.fromString(value)
+            font = QFont()
+            font.fromString(value)
+            self.guiFont = fontMatcher(font)
         else:
             font = QFont()
             fontDB = QFontDatabase()
@@ -377,11 +399,9 @@ class Config:
                 font.setPointSize(10)
             else:
                 font = fontDB.systemFont(QFontDatabase.SystemFont.GeneralFont)
-            self.guiFont = font
+            self.guiFont = fontMatcher(font)
             logger.debug("GUI font set to: %s", describeFont(font))
-
         QApplication.setFont(self.guiFont)
-
         return
 
     def setTextFont(self, value: QFont | str | None) -> None:
@@ -389,10 +409,11 @@ class Config:
         set to default font.
         """
         if isinstance(value, QFont):
-            self.textFont = value
+            self.textFont = fontMatcher(value)
         elif value and isinstance(value, str):
-            self.textFont = QFont()
-            self.textFont.fromString(value)
+            font = QFont()
+            font.fromString(value)
+            self.textFont = fontMatcher(font)
         else:
             fontDB = QFontDatabase()
             fontFam = fontDB.families()
@@ -406,8 +427,8 @@ class Config:
                 font.setPointSize(12)
             else:
                 font = fontDB.systemFont(QFontDatabase.SystemFont.GeneralFont)
-            self.textFont = font
-            logger.debug("Text font set to: %s", describeFont(font))
+            self.textFont = fontMatcher(font)
+            logger.debug("Text font set to: %s", describeFont(self.textFont))
         return
 
     ##
@@ -619,6 +640,7 @@ class Config:
         self._backupPath     = conf.rdPath(sec, "backuppath", self._backupPath)
         self.backupOnClose   = conf.rdBool(sec, "backuponclose", self.backupOnClose)
         self.askBeforeBackup = conf.rdBool(sec, "askbeforebackup", self.askBeforeBackup)
+        self.askBeforeExit   = conf.rdBool(sec, "askbeforeexit", self.askBeforeExit)
 
         # Editor
         sec = "Editor"
@@ -653,10 +675,11 @@ class Config:
         self.showFullPath    = conf.rdBool(sec, "showfullpath", self.showFullPath)
         self.dialogStyle     = conf.rdInt(sec, "dialogstyle", self.dialogStyle)
         self.allowOpenDial   = conf.rdBool(sec, "allowopendial", self.allowOpenDial)
+        self.dialogLine      = conf.rdStr(sec, "dialogline", self.dialogLine)
         self.narratorBreak   = conf.rdStr(sec, "narratorbreak", self.narratorBreak)
+        self.narratorDialog  = conf.rdStr(sec, "narratordialog", self.narratorDialog)
         self.altDialogOpen   = conf.rdStr(sec, "altdialogopen", self.altDialogOpen)
         self.altDialogClose  = conf.rdStr(sec, "altdialogclose", self.altDialogClose)
-        self.dialogLine      = conf.rdStr(sec, "dialogline", self.dialogLine)
         self.highlightEmph   = conf.rdBool(sec, "highlightemph", self.highlightEmph)
         self.stopWhenIdle    = conf.rdBool(sec, "stopwhenidle", self.stopWhenIdle)
         self.userIdleTime    = conf.rdInt(sec, "useridletime", self.userIdleTime)
@@ -665,6 +688,7 @@ class Config:
         sec = "State"
         self.showViewerPanel = conf.rdBool(sec, "showviewerpanel", self.showViewerPanel)
         self.showEditToolBar = conf.rdBool(sec, "showedittoolbar", self.showEditToolBar)
+        self.showSessionTime = conf.rdBool(sec, "showsessiontime", self.showSessionTime)
         self.viewComments    = conf.rdBool(sec, "viewcomments", self.viewComments)
         self.viewSynopsis    = conf.rdBool(sec, "viewsynopsis", self.viewSynopsis)
         self.searchCase      = conf.rdBool(sec, "searchcase", self.searchCase)
@@ -728,6 +752,7 @@ class Config:
             "backuppath":      str(self._backupPath),
             "backuponclose":   str(self.backupOnClose),
             "askbeforebackup": str(self.askBeforeBackup),
+            "askbeforeexit":   str(self.askBeforeExit),
         }
 
         conf["Editor"] = {
@@ -762,10 +787,11 @@ class Config:
             "showfullpath":    str(self.showFullPath),
             "dialogstyle":     str(self.dialogStyle),
             "allowopendial":   str(self.allowOpenDial),
+            "dialogline":      str(self.dialogLine),
             "narratorbreak":   str(self.narratorBreak),
+            "narratordialog":  str(self.narratorDialog),
             "altdialogopen":   str(self.altDialogOpen),
             "altdialogclose":  str(self.altDialogClose),
-            "dialogline":      str(self.dialogLine),
             "highlightemph":   str(self.highlightEmph),
             "stopwhenidle":    str(self.stopWhenIdle),
             "useridletime":    str(self.userIdleTime),
@@ -774,6 +800,7 @@ class Config:
         conf["State"] = {
             "showviewerpanel": str(self.showViewerPanel),
             "showedittoolbar": str(self.showEditToolBar),
+            "showsessiontime": str(self.showSessionTime),
             "viewcomments":    str(self.viewComments),
             "viewsynopsis":    str(self.viewSynopsis),
             "searchcase":      str(self.searchCase),
@@ -829,29 +856,30 @@ class RecentProjects:
 
     def __init__(self, config: Config) -> None:
         self._conf = config
-        self._data = {}
+        self._data: dict[str, dict[str, str | int]] = {}
+        self._map: dict[str, str] = {}
         return
 
     def loadCache(self) -> bool:
         """Load the cache file for recent projects."""
         self._data = {}
-
+        self._map = {}
         cacheFile = self._conf.dataPath(nwFiles.RECENT_FILE)
         if cacheFile.is_file():
             try:
                 with open(cacheFile, mode="r", encoding="utf-8") as inFile:
                     data = json.load(inFile)
                 for path, entry in data.items():
-                    self._data[path] = {
-                        "title": entry.get("title", ""),
-                        "words": entry.get("words", 0),
-                        "time": entry.get("time", 0),
-                    }
+                    puuid = str(entry.get("uuid", ""))
+                    title = str(entry.get("title", ""))
+                    words = checkInt(entry.get("words", 0), 0)
+                    saved = checkInt(entry.get("time", 0), 0)
+                    if path and title:
+                        self._setEntry(puuid, path, title, words, saved)
             except Exception:
                 logger.error("Could not load recent project cache")
                 logException()
                 return False
-
         return True
 
     def saveCache(self) -> bool:
@@ -866,7 +894,6 @@ class RecentProjects:
             logger.error("Could not save recent project cache")
             logException()
             return False
-
         return True
 
     def listEntries(self) -> list[tuple[str, str, int, int]]:
@@ -876,14 +903,15 @@ class RecentProjects:
             for k, e in self._data.items()
         ]
 
-    def update(self, path: str | Path, title: str, words: int, saved: float | int) -> None:
+    def update(self, path: str | Path, data: NWProjectData, saved: float | int) -> None:
         """Add or update recent cache information on a given project."""
-        self._data[str(path)] = {
-            "title": title,
-            "words": int(words),
-            "time": int(saved),
-        }
-        self.saveCache()
+        try:
+            if (remove := self._map.get(data.uuid)) and (remove != str(path)):
+                self.remove(remove)
+            self._setEntry(data.uuid, str(path), data.name, sum(data.currCounts), int(saved))
+            self.saveCache()
+        except Exception:
+            pass
         return
 
     def remove(self, path: str | Path) -> None:
@@ -893,6 +921,13 @@ class RecentProjects:
             self.saveCache()
         return
 
+    def _setEntry(self, puuid: str, path: str, title: str, words: int, saved: int) -> None:
+        """Set an entry in the recent projects record."""
+        self._data[path] = {"uuid": puuid, "title": title, "words": words, "time": saved}
+        if puuid:
+            self._map[puuid] = path
+        return
+
 
 class RecentPaths:
 
