@@ -6,7 +6,7 @@ File History:
 Created: 2018-10-27 [0.0.1] NWItem
 
 This file is a part of novelWriter
-Copyright 2018–2024, Veronica Berglyd Olsen
+Copyright (C) 2018 Veronica Berglyd Olsen and novelWriter contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -27,13 +27,14 @@ import logging
 
 from typing import TYPE_CHECKING, Any
 
-from PyQt5.QtGui import QIcon
+from PyQt5.QtGui import QFont, QIcon
 
+from novelwriter import CONFIG, SHARED
 from novelwriter.common import (
     checkInt, isHandle, isItemClass, isItemLayout, isItemType, simplified,
     yesNo
 )
-from novelwriter.constants import nwHeaders, nwLabels, trConst
+from novelwriter.constants import nwLabels, nwStyles, trConst
 from novelwriter.enum import nwItemClass, nwItemLayout, nwItemType
 
 if TYPE_CHECKING:  # pragma: no cover
@@ -256,6 +257,8 @@ class NWItem:
             self._paraCount = 0
             self._cursorPos = 0
 
+        self._initCount = self._wordCount
+
         return True
 
     @classmethod
@@ -281,6 +284,15 @@ class NWItem:
         cls._initCount = source._initCount
         return cls
 
+    ##
+    #  Action Methods
+    ##
+
+    def notifyToRefresh(self) -> None:
+        """Notify GUI that item info needs to be refreshed."""
+        self._project.tree.refreshItems([self._handle])
+        return
+
     ##
     #  Lookup Methods
     ##
@@ -309,6 +321,19 @@ class NWItem:
 
         return trConst(nwLabels.ITEM_DESCRIPTION.get(descKey, ""))
 
+    def getMainIcon(self) -> QIcon:
+        """Get the main item icon."""
+        return SHARED.theme.getItemIcon(self._type, self._class, self._layout, self._heading)
+
+    def getMainFont(self) -> QFont:
+        """Get the main item icon."""
+        if CONFIG.emphLabels and self._layout == nwItemLayout.DOCUMENT:
+            if self._heading == "H1":
+                return SHARED.theme.guiFontBU
+            elif self._heading == "H2":
+                return SHARED.theme.guiFontB
+        return SHARED.theme.guiFont
+
     def getImportStatus(self) -> tuple[str, QIcon]:
         """Return the relevant importance or status label and icon for
         the current item based on its class.
@@ -319,6 +344,19 @@ class NWItem:
             entry = self._project.data.itemImport[self._import]
         return entry.name, entry.icon
 
+    def getActiveStatus(self) -> tuple[str, QIcon]:
+        """Return the relevant active status label and icon for
+        the current item based on its type.
+        """
+        if self.isFileType():
+            key = "checked" if self._active else "unchecked"
+            text = trConst(nwLabels.ACTIVE_NAME[key])
+            icon = SHARED.theme.getIcon(key)
+        else:
+            text = ""
+            icon = SHARED.theme.getIcon("noncheckable")
+        return text, icon
+
     ##
     #  Checker Methods
     ##
@@ -518,7 +556,7 @@ class NWItem:
 
     def setMainHeading(self, value: str) -> None:
         """Set the main heading level."""
-        if value in nwHeaders.H_LEVEL:
+        if value in nwStyles.H_LEVEL:
             self._heading = value
         return
 
@@ -553,8 +591,3 @@ class NWItem:
         else:
             self._cursorPos = 0
         return
-
-    def saveInitialCount(self) -> None:
-        """Save the initial word count."""
-        self._initCount = self._wordCount
-        return
