@@ -7,7 +7,7 @@ Created:   2019-10-21 [0.3.1] OptionState
 Rewritten: 2020-02-19 [0.4.5] OptionState
 
 This file is a part of novelWriter
-Copyright 2018–2024, Veronica Berglyd Olsen
+Copyright (C) 2019 Veronica Berglyd Olsen and novelWriter contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -60,7 +60,7 @@ VALID_MAP: dict[str, set[str]] = {
     },
     "GuiManuscript": {
         "winWidth", "winHeight", "optsWidth", "viewWidth", "listHeight",
-        "detailsHeight", "detailsWidth", "detailsExpanded",
+        "detailsHeight", "detailsWidth", "detailsExpanded", "showNewPage",
     },
     "GuiManuscriptBuild": {
         "winWidth", "winHeight", "fmtWidth", "sumWidth",
@@ -73,6 +73,9 @@ VALID_MAP: dict[str, set[str]] = {
         "widthCol3", "widthCol4", "wordsPerPage", "countFrom", "clearDouble",
         "novelRoot",
     },
+    "GuiOutlineDetails": {
+        "detailsWidth", "tagsWidth",
+    }
 }
 
 
