@@ -7,7 +7,7 @@ Created: 2019-06-11 [0.1.5] NWSpellEnchant
 Created: 2023-06-13 [2.1b1] UserDictionary
 
 This file is a part of novelWriter
-Copyright 2018–2024, Veronica Berglyd Olsen
+Copyright (C) 2019 Veronica Berglyd Olsen and novelWriter contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -125,21 +125,16 @@ class NWSpellEnchant:
         except Exception:
             return []
 
-    def addWord(self, word: str) -> bool:
+    def addWord(self, word: str, save: bool = True) -> None:
         """Add a word to the project dictionary."""
-        word = word.strip()
-        if not word:
-            return False
-        try:
-            self._enchant.add_to_session(word)
-        except Exception:
-            return False
-
-        added = self._userDict.add(word)
-        if added:
-            self._userDict.save()
-
-        return added
+        if word := word.strip():
+            try:
+                self._enchant.add_to_session(word)
+            except Exception:
+                return
+            if save and self._userDict.add(word):
+                self._userDict.save()
+        return
 
     def listDictionaries(self) -> list[tuple[str, str]]:
         """List available dictionaries."""
