@@ -7,7 +7,7 @@ Created:   2019-06-10 [0.1.5] GuiPreferences
 Rewritten: 2024-01-08 [2.3b1] GuiPreferences
 
 This file is a part of novelWriter
-Copyright 2018–2024, Veronica Berglyd Olsen
+Copyright (C) 2019 Veronica Berglyd Olsen and novelWriter contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ from PyQt5.QtWidgets import (
 )
 
 from novelwriter import CONFIG, SHARED
-from novelwriter.common import describeFont, uniqueCompact
+from novelwriter.common import compact, describeFont, uniqueCompact
 from novelwriter.constants import nwUnicode
 from novelwriter.dialogs.quotes import GuiQuoteSelect
 from novelwriter.extensions.configlayout import NColourLabel, NScrollableForm
@@ -133,7 +133,7 @@ class GuiPreferences(NDialog):
         """Build the settings form."""
         section = 0
         iSz = SHARED.theme.baseIconSize
-        boxFixed = 5*SHARED.theme.textNWidth
+        boxFixed = 6*SHARED.theme.textNWidth
         minWidth = CONFIG.pxInt(200)
         fontWidth = CONFIG.pxInt(162)
 
@@ -271,10 +271,10 @@ class GuiPreferences(NDialog):
             self.tr("Include project notes in status bar word count"), self.incNotesWCount
         )
 
-        # Auto Save
+        # Behaviour
         # =========
 
-        title = self.tr("Auto Save")
+        title = self.tr("Behaviour")
         section += 1
         self.sidebar.addButton(title, section)
         self.mainForm.addGroupLabel(title, section)
@@ -301,6 +301,14 @@ class GuiPreferences(NDialog):
             self.tr("How often the project is automatically saved."), unit=self.tr("seconds")
         )
 
+        # Ask before exiting novelWriter
+        self.askBeforeExit = NSwitch(self)
+        self.askBeforeExit.setChecked(CONFIG.askBeforeExit)
+        self.mainForm.addRow(
+            self.tr("Ask before exiting novelWriter"), self.askBeforeExit,
+            self.tr("Only applies when a project is open.")
+        )
+
         # Project Backup
         # ==============
 
@@ -543,33 +551,6 @@ class GuiPreferences(NDialog):
             self.tr("Applies to the selected quote styles.")
         )
 
-        self.allowOpenDial = NSwitch(self)
-        self.allowOpenDial.setChecked(CONFIG.allowOpenDial)
-        self.mainForm.addRow(
-            self.tr("Allow open-ended dialogue"), self.allowOpenDial,
-            self.tr("Highlight dialogue line with no closing quote.")
-        )
-
-        self.narratorBreak = QLineEdit(self)
-        self.narratorBreak.setMaxLength(1)
-        self.narratorBreak.setFixedWidth(boxFixed)
-        self.narratorBreak.setAlignment(QtAlignCenter)
-        self.narratorBreak.setText(CONFIG.narratorBreak)
-        self.mainForm.addRow(
-            self.tr("Dialogue narrator break symbol"), self.narratorBreak,
-            self.tr("Symbol to indicate injected narrator break.")
-        )
-
-        self.dialogLine = QLineEdit(self)
-        self.dialogLine.setMaxLength(1)
-        self.dialogLine.setFixedWidth(boxFixed)
-        self.dialogLine.setAlignment(QtAlignCenter)
-        self.dialogLine.setText(CONFIG.dialogLine)
-        self.mainForm.addRow(
-            self.tr("Dialogue line symbol"), self.dialogLine,
-            self.tr("Lines starting with this symbol are dialogue.")
-        )
-
         self.altDialogOpen = QLineEdit(self)
         self.altDialogOpen.setMaxLength(4)
         self.altDialogOpen.setFixedWidth(boxFixed)
@@ -587,6 +568,43 @@ class GuiPreferences(NDialog):
             self.tr("Custom highlighting of dialogue text.")
         )
 
+        self.allowOpenDial = NSwitch(self)
+        self.allowOpenDial.setChecked(CONFIG.allowOpenDial)
+        self.mainForm.addRow(
+            self.tr("Allow open-ended dialogue"), self.allowOpenDial,
+            self.tr("Highlight dialogue line with no closing quote.")
+        )
+
+        self.dialogLine = QLineEdit(self)
+        self.dialogLine.setMaxLength(4)
+        self.dialogLine.setFixedWidth(boxFixed)
+        self.dialogLine.setAlignment(QtAlignCenter)
+        self.dialogLine.setText(CONFIG.dialogLine)
+        self.mainForm.addRow(
+            self.tr("Dialogue line symbols"), self.dialogLine,
+            self.tr("Lines starting with any of these symbols are dialogue.")
+        )
+
+        self.narratorBreak = QLineEdit(self)
+        self.narratorBreak.setMaxLength(1)
+        self.narratorBreak.setFixedWidth(boxFixed)
+        self.narratorBreak.setAlignment(QtAlignCenter)
+        self.narratorBreak.setText(CONFIG.narratorBreak)
+        self.mainForm.addRow(
+            self.tr("Narrator break symbol"), self.narratorBreak,
+            self.tr("Symbol to indicate a narrator break in dialogue.")
+        )
+
+        self.narratorDialog = QLineEdit(self)
+        self.narratorDialog.setMaxLength(1)
+        self.narratorDialog.setFixedWidth(boxFixed)
+        self.narratorDialog.setAlignment(QtAlignCenter)
+        self.narratorDialog.setText(CONFIG.narratorDialog)
+        self.mainForm.addRow(
+            self.tr("Alternating dialogue/narration symbol"), self.narratorDialog,
+            self.tr("Alternates dialogue highlighting within any paragraph.")
+        )
+
         self.highlightEmph = NSwitch(self)
         self.highlightEmph.setChecked(CONFIG.highlightEmph)
         self.mainForm.addRow(
@@ -917,9 +935,10 @@ class GuiPreferences(NDialog):
         CONFIG.incNotesWCount = self.incNotesWCount.isChecked()
         CONFIG.setTextFont(self._textFont)
 
-        # Auto Save
-        CONFIG.autoSaveDoc  = self.autoSaveDoc.value()
-        CONFIG.autoSaveProj = self.autoSaveProj.value()
+        # Behaviour
+        CONFIG.autoSaveDoc   = self.autoSaveDoc.value()
+        CONFIG.autoSaveProj  = self.autoSaveProj.value()
+        CONFIG.askBeforeExit = self.askBeforeExit.isChecked()
 
         # Project Backup
         CONFIG.setBackupPath(self.backupPath)
@@ -952,17 +971,19 @@ class GuiPreferences(NDialog):
         # Text Highlighting
         dialogueStyle   = self.dialogStyle.currentData()
         allowOpenDial   = self.allowOpenDial.isChecked()
+        dialogueLine    = uniqueCompact(self.dialogLine.text())
         narratorBreak   = self.narratorBreak.text().strip()
-        dialogueLine    = self.dialogLine.text().strip()
-        altDialogOpen   = self.altDialogOpen.text()
-        altDialogClose  = self.altDialogClose.text()
+        narratorDialog  = self.narratorDialog.text().strip()
+        altDialogOpen   = compact(self.altDialogOpen.text())
+        altDialogClose  = compact(self.altDialogClose.text())
         highlightEmph   = self.highlightEmph.isChecked()
         showMultiSpaces = self.showMultiSpaces.isChecked()
 
         updateSyntax |= CONFIG.dialogStyle != dialogueStyle
         updateSyntax |= CONFIG.allowOpenDial != allowOpenDial
-        updateSyntax |= CONFIG.narratorBreak != narratorBreak
         updateSyntax |= CONFIG.dialogLine != dialogueLine
+        updateSyntax |= CONFIG.narratorBreak != narratorBreak
+        updateSyntax |= CONFIG.narratorDialog != narratorDialog
         updateSyntax |= CONFIG.altDialogOpen != altDialogOpen
         updateSyntax |= CONFIG.altDialogClose != altDialogClose
         updateSyntax |= CONFIG.highlightEmph != highlightEmph
@@ -970,8 +991,9 @@ class GuiPreferences(NDialog):
 
         CONFIG.dialogStyle     = dialogueStyle
         CONFIG.allowOpenDial   = allowOpenDial
-        CONFIG.narratorBreak   = narratorBreak
         CONFIG.dialogLine      = dialogueLine
+        CONFIG.narratorBreak   = narratorBreak
+        CONFIG.narratorDialog  = narratorDialog
         CONFIG.altDialogOpen   = altDialogOpen
         CONFIG.altDialogClose  = altDialogClose
         CONFIG.highlightEmph   = highlightEmph
