@@ -6,7 +6,7 @@ File History:
 Created: 2022-11-17 [2.0] NovelSelector
 
 This file is a part of novelWriter
-Copyright 2018–2024, Veronica Berglyd Olsen
+Copyright (C) 2022 Veronica Berglyd Olsen and novelWriter contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -90,12 +90,13 @@ class NovelSelector(QComboBox):
     @pyqtSlot()
     def refreshNovelList(self) -> None:
         """Rebuild the list of novel items."""
+        cHandle = self.currentData()
+
         self._blockSignal = True
         self._firstHandle = None
         self.clear()
 
         icon = SHARED.theme.getIcon(nwLabels.CLASS_ICON[nwItemClass.NOVEL])
-        handle = self.currentData()
         for tHandle, nwItem in SHARED.project.tree.iterRoots(nwItemClass.NOVEL):
             if self._listFormat:
                 name = self._listFormat.format(nwItem.itemName)
@@ -110,7 +111,7 @@ class NovelSelector(QComboBox):
             self.insertSeparator(self.count())
             self.addItem(icon, self.tr("All Novel Folders"), "")
 
-        self.setHandle(handle)
+        self.setHandle(cHandle)
         self.setEnabled(self.count() > 1)
         self._blockSignal = False
 
