@@ -9,7 +9,7 @@ Created: 2020-06-09 [0.8]   GuiDocViewFooter
 Created: 2020-09-08 [1.0b1] GuiDocViewHistory
 
 This file is a part of novelWriter
-Copyright 2018–2024, Veronica Berglyd Olsen
+Copyright (C) 2019 Veronica Berglyd Olsen and novelWriter contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -31,34 +31,43 @@ import logging
 from enum import Enum
 
 from PyQt5.QtCore import QPoint, Qt, QUrl, pyqtSignal, pyqtSlot
-from PyQt5.QtGui import QCursor, QMouseEvent, QPalette, QResizeEvent, QTextCursor
+from PyQt5.QtGui import (
+    QCursor, QDesktopServices, QDragEnterEvent, QDragMoveEvent, QDropEvent,
+    QMouseEvent, QPalette, QResizeEvent, QTextCursor
+)
 from PyQt5.QtWidgets import (
     QAction, QApplication, QFrame, QHBoxLayout, QMenu, QTextBrowser,
     QToolButton, QWidget
 )
 
 from novelwriter import CONFIG, SHARED
-from novelwriter.constants import nwHeaders, nwUnicode
-from novelwriter.core.toqdoc import TextDocumentTheme, ToQTextDocument
-from novelwriter.enum import nwDocAction, nwDocMode, nwItemType
+from novelwriter.common import decodeMimeHandles, qtLambda
+from novelwriter.constants import nwConst, nwStyles, nwUnicode
+from novelwriter.enum import nwChange, nwDocAction, nwDocMode, nwItemType
 from novelwriter.error import logException
 from novelwriter.extensions.configlayout import NColourLabel
 from novelwriter.extensions.eventfilters import WheelEventFilter
 from novelwriter.extensions.modified import NIconToolButton
+from novelwriter.formats.shared import TextDocumentTheme
+from novelwriter.formats.toqdoc import ToQTextDocument
 from novelwriter.gui.theme import STYLES_MIN_TOOLBUTTON
-from novelwriter.types import QtAlignCenterTop, QtKeepAnchor, QtMouseLeft, QtMoveAnchor
+from novelwriter.types import (
+    QtAlignCenterTop, QtKeepAnchor, QtMouseLeft, QtMoveAnchor,
+    QtScrollAlwaysOff, QtScrollAsNeeded
+)
 
 logger = logging.getLogger(__name__)
 
 
 class GuiDocViewer(QTextBrowser):
 
+    closeDocumentRequest = pyqtSignal()
     documentLoaded = pyqtSignal(str)
     loadDocumentTagRequest = pyqtSignal(str, Enum)
-    closeDocumentRequest = pyqtSignal()
+    openDocumentRequest = pyqtSignal(str, Enum, str, bool)
     reloadDocumentRequest = pyqtSignal()
-    togglePanelVisibility = pyqtSignal()
     requestProjectItemSelected = pyqtSignal(str, bool)
+    togglePanelVisibility = pyqtSignal()
 
     def __init__(self, parent: QWidget) -> None:
         super().__init__(parent=parent)
@@ -72,6 +81,7 @@ class GuiDocViewer(QTextBrowser):
         # Settings
         self.setMinimumWidth(CONFIG.pxInt(300))
         self.setAutoFillBackground(True)
+        self.setOpenLinks(False)
         self.setOpenExternalLinks(False)
         self.setFocusPolicy(Qt.FocusPolicy.StrongFocus)
         self.setFrameStyle(QFrame.Shape.NoFrame)
@@ -162,6 +172,7 @@ class GuiDocViewer(QTextBrowser):
         self._docTheme.text      = SHARED.theme.colText
         self._docTheme.highlight = SHARED.theme.colMark
         self._docTheme.head      = SHARED.theme.colHead
+        self._docTheme.link      = SHARED.theme.colLink
         self._docTheme.comment   = SHARED.theme.colHidden
         self._docTheme.note      = SHARED.theme.colNote
         self._docTheme.code      = SHARED.theme.colCode
@@ -177,14 +188,14 @@ class GuiDocViewer(QTextBrowser):
 
         # Scroll bars
         if CONFIG.hideVScroll:
-            self.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
+            self.setVerticalScrollBarPolicy(QtScrollAlwaysOff)
         else:
-            self.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
+            self.setVerticalScrollBarPolicy(QtScrollAsNeeded)
 
         if CONFIG.hideHScroll:
-            self.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
+            self.setHorizontalScrollBarPolicy(QtScrollAlwaysOff)
         else:
-            self.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
+            self.setHorizontalScrollBarPolicy(QtScrollAsNeeded)
 
         # Refresh the tab stops
         self.setTabStopDistance(CONFIG.getTabWidth())
@@ -207,8 +218,10 @@ class GuiDocViewer(QTextBrowser):
         sPos = self.verticalScrollBar().value()
         qDoc = ToQTextDocument(SHARED.project)
         qDoc.setJustify(CONFIG.doJustify)
-        qDoc.setDialogueHighlight(True)
-        qDoc.initDocument(CONFIG.textFont, self._docTheme)
+        qDoc.setDialogHighlight(True)
+        qDoc.setTextFont(CONFIG.textFont)
+        qDoc.setTheme(self._docTheme)
+        qDoc.initDocument()
         qDoc.setKeywords(True)
         qDoc.setComments(CONFIG.viewComments)
         qDoc.setSynopsis(CONFIG.viewSynopsis)
@@ -221,7 +234,7 @@ class GuiDocViewer(QTextBrowser):
             qDoc.doPreProcessing()
             qDoc.tokenizeText()
             qDoc.doConvert()
-            qDoc.appendFootnotes()
+            qDoc.closeDocument()
         except Exception:
             logger.error("Failed to generate preview for document with handle '%s'", tHandle)
             logException()
@@ -245,7 +258,7 @@ class GuiDocViewer(QTextBrowser):
         SHARED.project.data.setLastHandle(tHandle, "viewer")
         self.docHeader.setHandle(tHandle)
         self.docHeader.setOutline({
-            sTitle: (hItem.title, nwHeaders.H_LEVEL.get(hItem.level, 0))
+            sTitle: (hItem.title, nwStyles.H_LEVEL.get(hItem.level, 0))
             for sTitle, hItem in SHARED.project.index.iterItemHeadings(tHandle)
         })
         self.updateDocMargins()
@@ -333,10 +346,10 @@ class GuiDocViewer(QTextBrowser):
     #  Public Slots
     ##
 
-    @pyqtSlot(str)
-    def updateDocInfo(self, tHandle: str) -> None:
+    @pyqtSlot(str, Enum)
+    def onProjectItemChanged(self, tHandle: str, change: nwChange) -> None:
         """Update the header title bar if needed."""
-        if tHandle and tHandle == self._docHandle:
+        if tHandle == self._docHandle and change == nwChange.UPDATE:
             self.docHeader.setHandle(tHandle)
             self.updateDocMargins()
         return
@@ -372,8 +385,10 @@ class GuiDocViewer(QTextBrowser):
             logger.debug("Clicked link: '%s'", link)
             if (bits := link.partition("_")) and bits[0] == "#tag" and bits[2]:
                 self.loadDocumentTagRequest.emit(bits[2], nwDocMode.VIEW)
-            else:
+            elif link.startswith("#"):
                 self.navigateTo(link)
+            elif link.startswith("http"):
+                QDesktopServices.openUrl(QUrl(url))
         return
 
     @pyqtSlot("QPoint")
@@ -386,30 +401,30 @@ class GuiDocViewer(QTextBrowser):
 
         if userSelection:
             mnuCopy = QAction(self.tr("Copy"), ctxMenu)
-            mnuCopy.triggered.connect(lambda: self.docAction(nwDocAction.COPY))
+            mnuCopy.triggered.connect(qtLambda(self.docAction, nwDocAction.COPY))
             ctxMenu.addAction(mnuCopy)
 
             ctxMenu.addSeparator()
 
         mnuSelAll = QAction(self.tr("Select All"), ctxMenu)
-        mnuSelAll.triggered.connect(lambda: self.docAction(nwDocAction.SEL_ALL))
+        mnuSelAll.triggered.connect(qtLambda(self.docAction, nwDocAction.SEL_ALL))
         ctxMenu.addAction(mnuSelAll)
 
         mnuSelWord = QAction(self.tr("Select Word"), ctxMenu)
-        mnuSelWord.triggered.connect(
-            lambda: self._makePosSelection(QTextCursor.SelectionType.WordUnderCursor, point)
-        )
+        mnuSelWord.triggered.connect(qtLambda(
+            self._makePosSelection, QTextCursor.SelectionType.WordUnderCursor, point
+        ))
         ctxMenu.addAction(mnuSelWord)
 
         mnuSelPara = QAction(self.tr("Select Paragraph"), ctxMenu)
-        mnuSelPara.triggered.connect(
-            lambda: self._makePosSelection(QTextCursor.SelectionType.BlockUnderCursor, point)
-        )
+        mnuSelPara.triggered.connect(qtLambda(
+            self._makePosSelection, QTextCursor.SelectionType.BlockUnderCursor, point
+        ))
         ctxMenu.addAction(mnuSelPara)
 
         # Open the context menu
         ctxMenu.exec(self.viewport().mapToGlobal(point))
-        ctxMenu.deleteLater()
+        ctxMenu.setParent(None)
 
         return
 
@@ -433,6 +448,32 @@ class GuiDocViewer(QTextBrowser):
             super().mouseReleaseEvent(event)
         return
 
+    def dragEnterEvent(self, event: QDragEnterEvent) -> None:
+        """Overload drag enter event to handle dragged items."""
+        if event.mimeData().hasFormat(nwConst.MIME_HANDLE):
+            event.acceptProposedAction()
+        else:
+            super().dragEnterEvent(event)
+        return
+
+    def dragMoveEvent(self, event: QDragMoveEvent) -> None:
+        """Overload drag move event to handle dragged items."""
+        if event.mimeData().hasFormat(nwConst.MIME_HANDLE):
+            event.acceptProposedAction()
+        else:
+            super().dragMoveEvent(event)
+        return
+
+    def dropEvent(self, event: QDropEvent) -> None:
+        """Overload drop event to handle dragged items."""
+        if event.mimeData().hasFormat(nwConst.MIME_HANDLE):
+            if handles := decodeMimeHandles(event.mimeData()):
+                if SHARED.project.tree.checkType(handles[0], nwItemType.FILE):
+                    self.openDocumentRequest.emit(handles[0], nwDocMode.VIEW, "", True)
+        else:
+            super().dropEvent(event)
+        return
+
     ##
     #  Internal Functions
     ##
@@ -629,6 +670,11 @@ class GuiDocViewHeader(QWidget):
         self.forwardButton.setToolTip(self.tr("Go Forward"))
         self.forwardButton.clicked.connect(self.docViewer.navForward)
 
+        self.editButton = NIconToolButton(self, iSz)
+        self.editButton.setVisible(False)
+        self.editButton.setToolTip(self.tr("Open in Editor"))
+        self.editButton.clicked.connect(self._editDocument)
+
         self.refreshButton = NIconToolButton(self, iSz)
         self.refreshButton.setVisible(False)
         self.refreshButton.setToolTip(self.tr("Reload"))
@@ -647,7 +693,7 @@ class GuiDocViewHeader(QWidget):
         self.outerBox.addSpacing(mPx)
         self.outerBox.addWidget(self.itemTitle, 1)
         self.outerBox.addSpacing(mPx)
-        self.outerBox.addSpacing(iPx)
+        self.outerBox.addWidget(self.editButton, 0)
         self.outerBox.addWidget(self.refreshButton, 0)
         self.outerBox.addWidget(self.closeButton, 0)
         self.outerBox.setSpacing(0)
@@ -681,8 +727,9 @@ class GuiDocViewHeader(QWidget):
         self.outlineButton.setVisible(False)
         self.backButton.setVisible(False)
         self.forwardButton.setVisible(False)
-        self.closeButton.setVisible(False)
+        self.editButton.setVisible(False)
         self.refreshButton.setVisible(False)
+        self.closeButton.setVisible(False)
         return
 
     def setOutline(self, data: dict[str, tuple[str, int]]) -> None:
@@ -716,6 +763,7 @@ class GuiDocViewHeader(QWidget):
         self.outlineButton.setThemeIcon("list")
         self.backButton.setThemeIcon("backward")
         self.forwardButton.setThemeIcon("forward")
+        self.editButton.setThemeIcon("edit")
         self.refreshButton.setThemeIcon("refresh")
         self.closeButton.setThemeIcon("close")
 
@@ -723,6 +771,7 @@ class GuiDocViewHeader(QWidget):
         self.outlineButton.setStyleSheet(buttonStyle)
         self.backButton.setStyleSheet(buttonStyle)
         self.forwardButton.setStyleSheet(buttonStyle)
+        self.editButton.setStyleSheet(buttonStyle)
         self.refreshButton.setStyleSheet(buttonStyle)
         self.closeButton.setStyleSheet(buttonStyle)
 
@@ -757,7 +806,7 @@ class GuiDocViewHeader(QWidget):
 
         if CONFIG.showFullPath:
             self.itemTitle.setText(f"  {nwUnicode.U_RSAQUO}  ".join(reversed(
-                [name for name in SHARED.project.tree.getItemPath(tHandle, asName=True)]
+                [name for name in SHARED.project.tree.itemPath(tHandle, asName=True)]
             )))
         else:
             self.itemTitle.setText(i.itemName if (i := SHARED.project.tree[tHandle]) else "")
@@ -765,8 +814,9 @@ class GuiDocViewHeader(QWidget):
         self.backButton.setVisible(True)
         self.forwardButton.setVisible(True)
         self.outlineButton.setVisible(True)
-        self.closeButton.setVisible(True)
+        self.editButton.setVisible(True)
         self.refreshButton.setVisible(True)
+        self.closeButton.setVisible(True)
 
         return
 
@@ -793,6 +843,13 @@ class GuiDocViewHeader(QWidget):
         self.docViewer.reloadDocumentRequest.emit()
         return
 
+    @pyqtSlot()
+    def _editDocument(self) -> None:
+        """Open the document in the editor."""
+        if tHandle := self._docHandle:
+            self.docViewer.openDocumentRequest.emit(tHandle, nwDocMode.EDIT, "", True)
+        return
+
     ##
     #  Events
     ##
