@@ -6,7 +6,7 @@ File History:
 Created: 2023-09-07 [2.2b1] GuiTextDocument
 
 This file is a part of novelWriter
-Copyright 2018–2024, Veronica Berglyd Olsen
+Copyright (C) 2023 Veronica Berglyd Olsen and novelWriter contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -95,6 +95,21 @@ class GuiTextDocument(QTextDocument):
 
         return
 
+    def metaDataAtPos(self, pos: int) -> tuple[str, str]:
+        """Check if there is meta data available at a given position in
+        the document, and if so, return it.
+        """
+        cursor = QTextCursor(self)
+        cursor.setPosition(pos)
+        block = cursor.block()
+        data = block.userData()
+        if block.isValid() and isinstance(data, TextBlockData):
+            if (check := pos - block.position()) >= 0:
+                for cPos, cEnd, cData, cType in data.metaData:
+                    if cPos <= check <= cEnd:
+                        return cData, cType
+        return "", ""
+
     def spellErrorAtPos(self, pos: int) -> tuple[str, int, int, list[str]]:
         """Check if there is a misspelled word at a given position in
         the document, and if so, return it.
@@ -107,8 +122,8 @@ class GuiTextDocument(QTextDocument):
             text = block.text()
             check = pos - block.position()
             if check >= 0:
-                for cPos, cLen in data.spellErrors:
-                    cEnd = cPos + cLen
+                for cPos, cEnd in data.spellErrors:
+                    cLen = cEnd - cPos
                     if cPos <= check <= cEnd:
                         word = text[cPos:cEnd]
                         return word, cPos, cLen, SHARED.spelling.suggestWords(word)
