@@ -6,7 +6,7 @@ File History:
 Created: 2023-11-19 [2.2rc1] GuiDictionaries
 
 This file is a part of novelWriter
-Copyright 2018–2024, Veronica Berglyd Olsen
+Copyright (C) 2023 Veronica Berglyd Olsen and novelWriter contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -65,12 +65,12 @@ class GuiDictionaries(NNonBlockingDialog):
         self.setMinimumHeight(CONFIG.pxInt(300))
 
         # Hunspell Dictionaries
-        foUrl = "https://www.freeoffice.com/en/download/dictionaries"
         loUrl = "https://extensions.libreoffice.org"
+        ooUrl = "https://extensions.openoffice.org"
         self.huInfo = QLabel("<br>".join([
             self.tr("Download a dictionary from one of the links, and add it below."),
-            f"&nbsp;\u203a <a href='{foUrl}'>{foUrl}</a>",
             f"&nbsp;\u203a <a href='{loUrl}'>{loUrl}</a>",
+            f"&nbsp;\u203a <a href='{ooUrl}'>{ooUrl}</a>",
         ]), self)
         self.huInfo.setOpenExternalLinks(True)
         self.huInfo.setWordWrap(True)
