@@ -6,7 +6,7 @@ File History:
 Created: 2023-12-14 [2.3b1] GuiWelcome
 
 This file is a part of novelWriter
-Copyright 2018–2024, Veronica Berglyd Olsen
+Copyright (C) 2023 Veronica Berglyd Olsen and novelWriter contributors
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -49,7 +49,7 @@ from novelwriter.extensions.configlayout import NWrappedWidgetBox
 from novelwriter.extensions.modified import NDialog, NIconToolButton, NSpinBox
 from novelwriter.extensions.switch import NSwitch
 from novelwriter.extensions.versioninfo import VersionInfoWidget
-from novelwriter.types import QtAlignLeft, QtAlignRightTop, QtSelected
+from novelwriter.types import QtAlignLeft, QtAlignRightTop, QtScrollAsNeeded, QtSelected
 
 logger = logging.getLogger(__name__)
 
@@ -297,7 +297,7 @@ class _OpenProjectPage(QWidget):
         self.selectedPath.addAction(self.aMissing, QLineEdit.ActionPosition.TrailingPosition)
 
         self.keyDelete = QShortcut(self)
-        self.keyDelete.setKey(Qt.Key.Key_Delete)
+        self.keyDelete.setKey("Del")
         self.keyDelete.activated.connect(self._deleteSelectedItem)
 
         # Assemble
@@ -376,7 +376,7 @@ class _OpenProjectPage(QWidget):
         action = ctxMenu.addAction(self.tr("Remove Project"))
         action.triggered.connect(self._deleteSelectedItem)
         ctxMenu.exec(self.mapToGlobal(pos))
-        ctxMenu.deleteLater()
+        ctxMenu.setParent(None)
         return
 
     ##
@@ -501,8 +501,8 @@ class _NewProjectPage(QWidget):
         self.scrollArea = QScrollArea(self)
         self.scrollArea.setWidget(self.projectForm)
         self.scrollArea.setWidgetResizable(True)
-        self.scrollArea.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
-        self.scrollArea.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
+        self.scrollArea.setHorizontalScrollBarPolicy(QtScrollAsNeeded)
+        self.scrollArea.setVerticalScrollBarPolicy(QtScrollAsNeeded)
 
         self.enterForm = self.projectForm.enterForm
 
